@@ -179,6 +179,21 @@ COLOR_PALETTE = [
     102,
 ]
 
+OPACITY_DARK = 200
+OPACITY_THIN = 50
+KEYPOINT_MIN_STROKE_WIDTH = 2
+POSE_ESTIMATION_MIN_STROKE_WIDTH = 7
+SEPARATOER = "@@@@@"
+
+EXPORT_IMAGE_WITH_ANNOTATIONS_SUPPORTED_IMAGE_TYPES = (
+    ".jpeg",
+    ".jpg",
+    ".png",
+    ".tif",
+    ".tiff",
+    ".bmp",
+)
+
 # under 512 MB. Actual size is 536870888 bytes, but to consider other attributes,
 # minus 888 bytes.
 # Because of V8's limitation, API only can accept the JSON string that length is
@@ -203,6 +218,7 @@ class AnnotationType(Enum):
     polygon = "polygon"
     keypoint = "keypoint"
     line = "line"
+    circle = "circle"
     segmentation = "segmentation"
     classification = "classification"
     pose_estimation = "pose_estimation"
