@@ -10,6 +10,8 @@ import math
 from fastlabel.const import AnnotationType
 import os
 
+from fastlabel.exceptions import FastLabelInvalidException
+
 # COCO
 
 
@@ -54,23 +56,23 @@ def to_coco(tasks: list, annotations: list = []) -> dict:
     }
 
 
-def __get_coco_skelton(keypoints: list) -> list:
-    keypoint_id_skelton_index_map = {}
+def __get_coco_skeleton(keypoints: list) -> list:
+    keypoint_id_skeleton_index_map = {}
     for index, keypoint in enumerate(keypoints, 1):
-        keypoint_id_skelton_index_map[keypoint["id"]] = index
+        keypoint_id_skeleton_index_map[keypoint["id"]] = index
 
-    skelton = []
-    filtered_skelton_indexes = []
+    skeleton = []
+    filtered_skeleton_indexes = []
     for keypoint in keypoints:
         id = keypoint["id"]
-        skelton_index = keypoint_id_skelton_index_map[id]
+        skeleton_index = keypoint_id_skeleton_index_map[id]
         edges = keypoint["edges"]
         for edge in edges:
-            edge_skelton_index = keypoint_id_skelton_index_map[edge]
-            if not edge_skelton_index in filtered_skelton_indexes:
-                skelton.append([skelton_index, edge_skelton_index])
-            filtered_skelton_indexes.append(skelton_index)
-    return skelton
+            edge_skeleton_index = keypoint_id_skeleton_index_map[edge]
+            if not edge_skeleton_index in filtered_skeleton_indexes:
+                skeleton.append([skeleton_index, edge_skeleton_index])
+            filtered_skeleton_indexes.append(skeleton_index)
+    return skeleton
 
 
 def __get_categories(tasks: list, annotations: list) -> list:
@@ -87,7 +89,7 @@ def __get_categories(tasks: list, annotations: list) -> list:
     if not annotations:
         for index, value in enumerate(values, 1):
             category = {
-                "skelton": [],
+                "skeleton": [],
                 "keypoints": [],
                 "keypoint_colors": [],
                 "color": task_annotation["color"],
@@ -103,7 +105,7 @@ def __get_categories(tasks: list, annotations: list) -> list:
     for annotation in annotations:
         if not annotation["value"] in values:
             continue
-        coco_skelton = []
+        coco_skeleton = []
         coco_keypoints = []
         coco_keypoint_colors = []
         if annotation["type"] == AnnotationType.pose_estimation.value:
@@ -111,8 +113,8 @@ def __get_categories(tasks: list, annotations: list) -> list:
             for keypoint in keypoints:
                 coco_keypoints.append(keypoint["key"])
                 coco_keypoint_colors.append(keypoint["color"])
-            coco_skelton = __get_coco_skelton(keypoints)
-        category = {"skelton": coco_skelton,
+            coco_skeleton = __get_coco_skeleton(keypoints)
+        category = {"skeleton": coco_skeleton,
                     "keypoints": coco_keypoints,
                     "keypoint_colors": coco_keypoint_colors,
                     "color": annotation["color"],
@@ -373,7 +375,7 @@ def __get_yolo_annotation(data: dict) -> dict:
     dw = 1. / task["width"]
     dh = 1. / task["height"]
 
-    bbox = __to_bbox(points)
+    bbox = __to_bbox(annotation_type, points)
     xmin = bbox[0]
     ymin = bbox[1]
     xmax = bbox[0] + bbox[2]
@@ -443,7 +445,7 @@ def __get_pascalvoc_obj(data: dict) -> dict:
         return None
     if annotation_type == AnnotationType.bbox.value and (int(points[0]) == int(points[2]) or int(points[1]) == int(points[3])):
         return None
-    bbox = __to_bbox(points)
+    bbox = __to_bbox(annotation_type, points)
     x = bbox[0]
     y = bbox[1]
     w = bbox[2]
@@ -656,14 +658,16 @@ def __get_pixel_coordinates(points: List[int or float]) -> List[int]:
     return new_points
 
 
-def execute_coco_to_fastlabel(coco: dict) -> dict:
+def execute_coco_to_fastlabel(coco: dict ,annotation_type:str) -> dict:
     coco_images = {}
     for c in coco["images"]:
         coco_images[c["id"]] = c["file_name"]
 
     coco_categories = {}
+    coco_categories_keypoints={}
     for c in coco["categories"]:
-        coco_categories[c["id"]] = c["name"]
+        coco_categories[c["id"]] = c["supercategory"]
+        coco_categories_keypoints[c["id"]] = c["keypoints"]
 
     coco_annotations = coco["annotations"]
 
@@ -682,19 +686,51 @@ def execute_coco_to_fastlabel(coco: dict) -> dict:
             if not category_name:
                 return
 
-            segmentation = target_coco_annotation["segmentation"][0]
-            annotation_type = ""
-            if len(segmentation) == 4:
-                annotation_type = AnnotationType.bbox.value
-            if len(segmentation) > 4:
-                annotation_type = AnnotationType.polygon.value
-            annotations.append(
-                {
-                    "value": category_name,
-                    "points": segmentation,
-                    "type": annotation_type,
-                }
-            )
+            if annotation_type in [AnnotationType.bbox.value, AnnotationType.polygon.value]:
+                segmentation = target_coco_annotation["segmentation"][0]
+                annotation_type = ""
+                if len(segmentation) == 4:
+                    annotation_type = AnnotationType.bbox.value
+                if len(segmentation) > 4:
+                    annotation_type = AnnotationType.polygon.value
+                annotations.append(
+                    {
+                        "value": category_name,
+                        "points": segmentation,
+                        "type": annotation_type,
+                    }
+                )
+            elif annotation_type == AnnotationType.pose_estimation.value:
+                keypoints = []
+                target_coco_annotation_keypoints = target_coco_annotation["keypoints"]
+                keypoint_keys = coco_categories_keypoints[target_coco_annotation["category_id"]]
+                # coco keypoint style [100,200,1,300,400,1,500,600,2] convert to [[100,200,1],[300,400,1],[500,600,2]]
+                keypoint_values = [target_coco_annotation_keypoints[i:i + 3] for i in range(0, len(target_coco_annotation_keypoints), 3)]
+                for index, keypoint_key in enumerate(keypoint_keys):
+                    keypoint_value = keypoint_values[index]
+                    if keypoint_value[2] == 0:
+                        continue
+                    if not keypoint_value[2] in [1, 2]: 
+                        raise FastLabelInvalidException(f"Visibility flag must be 0 or 1, 2 . annotation_id: {target_coco_annotation['id']}", 422)
+                    # fastlabel occulusion is 0 or 1 . coco occulusion is 1 or 2. 
+                    keypoint_value[2] = keypoint_value[2] - 1
+                    keypoints.append({
+                        "key": keypoint_key,
+                        "value": keypoint_value
+                    })
+
+                annotations.append(
+                    {
+                        "value": category_name,
+                        "type": annotation_type,
+                        "keypoints": keypoints
+                    }
+                )
+            else:
+                raise FastLabelInvalidException(
+                "Annotation type must be bbox or polygon ,pose_estimation.", 422
+                )
+
         results[coco_images[coco_image_key]] = annotations
     return results
 
